(define-module (alterator d)
    :use-module (vhttpd)
    :use-module (srfi srfi-1)
    :use-module (alterator algo)
    :use-module (alterator pipe)
    :use-module (alterator woo)
    :use-module (alterator exit-handler)
    :export (d-loop
             d-control
             d-query

             d-init-local
             d-init-global
             d-end

             d))

;; safe send of scheme object
(define (write-sexp scm port)
  (if (or (pair? scm) (null? scm))
      (begin (write scm port)
             (force-output port))
      (error "wrong message" scm)))

(define *d-socket* "/var/run/alteratord/.socket")
(define *d-user* "root")
(define *d-group* "_alteratord")
(define *d-control* "/usr/lib/alteratord/control")
(define *d-process* #f)

(define *null-port* (open-output-file "/dev/null"))

(define (d-wait port timeout)
  (catch/ignore
    #t
    (lambda()
      (pair? (car (select (list (port->fdes port))
			  '()
			  '()
			  (and (positive? timeout) timeout)))))))

(define (d-loop-main port proc timeout)
  (and (d-wait port timeout)
       (let* ((s (car (accept port)))
	      (msg (read s)))
	 (or (eof-object? msg)
	     (write-sexp (proc msg) s))
	 (close-port s)
	 (d-loop-main port proc timeout))))

(define (d-loop proc timeout)
  (d-loop-main (make-unix-server-socket *d-socket* *d-user* *d-group*)
	       proc
	       timeout))

(define (d-control . args)
  (let ((pid (primitive-fork)))
    (cond
      ((zero? pid)
       (catch/ignore
	 #t
	 (lambda()
	   (dup2 (fileno *null-port*) 1)
	   (dup2 (fileno *null-port*) 2)
	   (apply execl *d-control* *d-control* args)))
       (primitive-exit 1))
      (else
	(waitpid pid)))))

(define *d-client-errno-list* (list ENOENT ECONNREFUSED))

(define (d-client-socket)
 (catch
  'system-error
  (lambda()
   (make-unix-client-socket *d-socket*))
  (lambda (key name fmt args errno-list)
   (if (and (string=? name "make-unix-client-socket")
            (member (car errno-list) *d-client-errno-list*))
       #f
       (error "unable to connect to alteratord socket")))))

(define (d-connect)
  (let ((port (d-client-socket)))
   (or port
       (begin (if *d-process*
		  (usleep 5000)
		  (d-control "start-nolock" "-t" "60"))
              (d-connect)))))

(define (d-query cmd)
  (let ((s (d-connect)))
    (write-sexp cmd s)
    (begin-1 (unpack-exception (read s))
             (close-port s))))

;;note: don't use control script, because init script pass all stderr to initlog

(define (d-init-local)
  (d-control "stop")
  (set! *d-process* (create-process #f "/usr/sbin/alteratord" "-f" "-l")))

(define (d-init-global)
  (d-control "stop")
  (set! *d-process* (create-process #f "/usr/sbin/alteratord" "-f")))

(define (d-end)
  (and *d-process*
       (begin (stop-process 'terminate *d-process*)
	      (set! *d-process* #f))))

;;backward compatibility: woobus-module
(define (d)
  (lambda(cmds next)
    (next (append-map d-query cmds))))

;;; at-exit cleanup
(at-exit d-end)
